// Sketcher3View.cpp : implementation of the CSketcher3View class
//

#include "stdafx.h"
#include "Sketcher3.h"

#include "Sketcher3Doc.h"
#include "Sketcher3View.h"

#include "element.h"
#include "constantes.h"
#include "line.h"
#include "cercle.h" 
#include "rectangle.h"
#include "curve.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSketcher3View

IMPLEMENT_DYNCREATE(CSketcher3View, CScrollView)

BEGIN_MESSAGE_MAP(CSketcher3View, CScrollView)
	//{{AFX_MSG_MAP(CSketcher3View)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDOWN()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSketcher3View construction/destruction

CSketcher3View::CSketcher3View()
{
	m_FirstPoint = CPoint(0,0) ;	// Le premier point est 0,0
	m_SecondPoint = CPoint(0,0) ;	// Le deuxime point est 0, 0
	m_pTempElement = 0 ;
}

CSketcher3View::~CSketcher3View()
{
}

BOOL CSketcher3View::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CSketcher3View drawing

void CSketcher3View::OnDraw(CDC* pDC)
{
	CSketcher3Doc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here

	// Position du premier element de la liste, boucler sur tous les elements
	POSITION aPos = pDoc->GetListHeadPosition() ;
	CElement* pElem = 0 ;
	while(aPos) {
		pElem = pDoc->GetNext(aPos) ;
		// Si l'element est visible on le dessine
		if(pDC->RectVisible(pElem->GetBoundRect())) pElem->Draw(pDC) ;
	}	
}

/////////////////////////////////////////////////////////////////////////////
// CSketcher3View printing

BOOL CSketcher3View::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CSketcher3View::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CSketcher3View::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CSketcher3View diagnostics

#ifdef _DEBUG
void CSketcher3View::AssertValid() const
{
	CView::AssertValid();
}

void CSketcher3View::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CSketcher3Doc* CSketcher3View::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CSketcher3Doc)));
	return (CSketcher3Doc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSketcher3View message handlers
void CSketcher3View::OnMouseMove(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	// Dfinit un objet de contexte de priphrique pour la vue
	CClientDC aDC(this) ;

	// Ajuste l'origine
	OnPrepareDC(&aDC) ;

	aDC.SetROP2(R2_NOTXORPEN) ;		// Dfinit le mode de dessin
	if( (nFlags & MK_LBUTTON) && (this==GetCapture()) )
	{
		// Convertit le point en coordonnes logiques
		aDC.DPtoLP(&point) ;

		// Enregistre la position en cours du curseur
		 m_SecondPoint = point ;		


		// Vrifier la prsence d'un lment temporaire
		if(m_pTempElement)
		{
			// est-ce une courbe ?
			if(GetDocument()->GetElementType() == CURVE) 
			{

				// Ajoute un segment  la courbe
				(static_cast<CCurve*>(m_pTempElement))->AddSegment(m_SecondPoint) ;
				
				// Trace l'mement
				m_pTempElement->Draw(&aDC) ;

				// Operation termine
				return ;
			}

			// Redessine l'ancien lment pour qu'il disparaisse de la vue
			m_pTempElement->Draw(&aDC) ;
			delete m_pTempElement ;		// Supprime l'ancien lment
			m_pTempElement = 0 ;		// Met le pointeur  0
		}

		// Cre un lment temporaire du type et de la couleur
		// Enregistrs dans l'objet document et le dessine
		m_pTempElement = CreateElement() ;
		m_pTempElement->Draw(&aDC) ;
	}
}

void CSketcher3View::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	if(this == GetCapture()) ReleaseCapture() ;

	// Verifie s'il y a un lment
	if(m_pTempElement) {
		// Appelle une fonction de la classe document pour stocker 
		// l'lment dsigner par m_pTempElement dans l'objet document
		GetDocument()->AddElement(m_pTempElement) ;
		InvalidateRect(0) ; // Redessine la fentre en cours
		
		//delete m_pTempElement ;
		m_pTempElement = 0 ;
	}
}

void CSketcher3View::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// Crer un contexte de priphrique
	CClientDC aDC(this) ;

	// Ajuste l'origine
	OnPrepareDC(&aDC) ;

	// Convertit le point en coordonnes logiques
	aDC.DPtoLP(&point) ;


	m_FirstPoint = point ;		// Enrigestre la position du curseur
	SetCapture() ;
}


// Cre un lment de type courant
CElement* CSketcher3View::CreateElement()
{
	// Obtient un pointeur dsignant le document associ  cette vue
	CSketcher3Doc*  pDoc = GetDocument() ;
	ASSERT_VALID(pDoc) ;	// Verifie que le pointeur est correct

	// Slectionne maintenant l'lment selon le type stock dans le document
	switch(pDoc->GetElementType())
	{
	case RECTANGLE:
		return new CRectangle(m_FirstPoint, m_SecondPoint, 
							 pDoc->GetElementWidth() ,
							 pDoc->GetElementColor()
							 ) ;
	case CERCLE:
		return new CCercle(m_FirstPoint, m_SecondPoint, 
							 pDoc->GetElementWidth() ,
							 pDoc->GetElementColor()
							 ) ;
	case CURVE:
		return new CCurve(m_FirstPoint, m_SecondPoint, 
							pDoc->GetElementWidth(), 
							pDoc->GetElementColor()
							) ;

	case LINE:
		return new CLine(m_FirstPoint, m_SecondPoint, 
						 pDoc->GetElementWidth(),
						 pDoc->GetElementColor()
						) ;
	default:
		// Il y a un problme
		AfxMessageBox("Code Incorrect", MB_OK) ;
		AfxAbort() ;
		return NULL ;
	}
}


void CSketcher3View::OnUpdate(CView *pSender, LPARAM LHint, CObject *pHint)
{

	// Invalide la zone correspendant  l'lmnt dsign
	// S'il y en a un, sinon invalide toute la zone client
	if(pHint) 
	{
		// Crer un contexte de priphrique
		CClientDC aDC(this) ;

		// Ajuste l'origine
		OnPrepareDC(&aDC) ;

		// Obtient le rectangle dlimiteur et 
		CRect aRect = static_cast<CElement*>(pHint)->GetBoundRect() ;
		
		// le convertir  rectangle dlimiteur en coordonnes client
		aDC.LPtoDP(aRect) ;

		// Invalide le rectangle pour le redessiner
		InvalidateRect(aRect) ;
	}
	else InvalidateRect(0) ;	// Invalide toute la zone client
}


void CSketcher3View::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate() ;
	
	// Definit la taille du document
	CSize DocSize(20000, 20000) ;

	// Dfinit le mode de trac et la taille du document
	SetScrollSizes(MM_TEXT, DocSize) ;
}

// -----------------------------------------------
// Recupre un pointeur sur la vue courante
// -----------------------------------------------
CSketcher3View * CSketcher3View::GetView()
{

      CFrameWnd * pFrame = (CFrameWnd *)(AfxGetApp()->m_pMainWnd);

      CView * pView = pFrame->GetActiveView();

      if ( !pView ) return NULL;

      // Fail if view is of wrong kind
      // (this could occur with splitter windows, or additional
      // views on a single document
      if ( ! pView->IsKindOf( RUNTIME_CLASS(CSketcher3View) ) )  return NULL;

      return (CSketcher3View *) pView;
}